@@ -1,6 +1,6 @@
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2018 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2019 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -541,6 +541,7 @@ set_option_space(struct dhcpcd_ctx *ctx,
 		return arg + strlen("nd_");
 	}
 
+#ifdef DHCP6
 	if (strncmp(arg, "dhcp6_", strlen("dhcp6_")) == 0) {
 		*d = ctx->dhcp6_opts;
 		*dl = ctx->dhcp6_opts_len;
@@ -553,6 +554,7 @@ set_option_space(struct dhcpcd_ctx *ctx,
 		return arg + strlen("dhcp6_");
 	}
 #endif
+#endif
 
 #ifdef INET
 	*d = ctx->dhcp_opts;
@@ -1360,6 +1362,7 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 		for (sl = 0; sl < ifo->ia_len; sl++) {
 			if ((arg == NULL && !ifo->ia[sl].iaid_set) ||
 			    (arg != NULL && ifo->ia[sl].iaid_set &&
+			    ifo->ia[sl].ia_type == (uint16_t)i &&
 			    ifo->ia[sl].iaid[0] == iaid[0] &&
 			    ifo->ia[sl].iaid[1] == iaid[1] &&
 			    ifo->ia[sl].iaid[2] == iaid[2] &&
@@ -1369,10 +1372,6 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 				break;
 			}
 		}
-		if (ia && ia->ia_type != (uint16_t)i) {
-			logerrx("Cannot mix IA for the same IAID");
-			break;
-		}
 		if (ia == NULL) {
 			ia = reallocarray(ifo->ia,
 			    ifo->ia_len + 1, sizeof(*ifo->ia));
@@ -1423,9 +1422,13 @@ parse_option(struct dhcpcd_ctx *ctx, const char *ifname, struct if_options *ifo,
 			ia->sla = NULL;
 #endif
 		}
+
+#ifdef SMALL
+		break;
+#else
 		if (ia->ia_type != D6_OPTION_IA_PD)
 			break;
-#ifndef SMALL
+
 		for (p = fp; p; p = fp) {
 			fp = strwhite(p);
 			if (fp) {
@@ -2013,7 +2016,7 @@ err_sla:
 		arg = fp;
 		fp = strend(arg);
 		if (fp == NULL) {
-			logerrx("authtoken requies an an expiry date");
+			logerrx("authtoken requies an expiry date");
 			free(token->realm);
 			free(token);
 			return -1;
@@ -2433,8 +2436,10 @@ read_config(struct dhcpcd_ctx *ctx,
 #ifdef INET6
 		ctx->nd_opts = ifo->nd_override;
 		ctx->nd_opts_len = ifo->nd_override_len;
+#ifdef DHCP6
 		ctx->dhcp6_opts = ifo->dhcp6_override;
 		ctx->dhcp6_opts_len = ifo->dhcp6_override_len;
+#endif
 #else
 		for (i = 0, opt = ifo->nd_override;
 		    i < ifo->nd_override_len;
